import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:http/http.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_payment/ocg_payment.dart';
import 'package:razorpay_flutter/razorpay_flutter.dart';

class PaymentRazorpayRepository extends PaymentRepository {
  static Razorpay _razorpay = Razorpay();
  static const String ERROR = "error";
  final String testKey;
  final String liveKey;
  final modeDebug;
  final Client client;

  final String paymentUrl = "/payments";

  PaymentRazorpayRepository(
    this.client, {
    this.testKey,
    this.liveKey,
    this.modeDebug = true,
  });

  String get razorpayKey => modeDebug ? testKey : liveKey;

  Future<AppResponse<PaymentResponse>> payAmount(PaymentRequest request) async {
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> addAmount(
    Amount amount,
  ) async {
    var placeResponse = await placeOrder(amount);
    if (!placeResponse.isSuccess) {
      return AppResponse.named(error: placeResponse.error);
    }
    Map<String, dynamic> options = {
      'key': razorpayKey,
      "order_id": placeResponse.data,
      'name': 'FairPriceIt',
      'description': 'Add money to wallet',
    };

    _razorpay.clear();
    var paymentResponse;
    _razorpay.on(Razorpay.EVENT_PAYMENT_SUCCESS,
        (PaymentSuccessResponse response) async {
      paymentResponse = response;
    });
    _razorpay.on(Razorpay.EVENT_PAYMENT_ERROR,
        (PaymentFailureResponse response) {
      paymentResponse = response;
      _razorpay.clear();
    });
    _razorpay.open(options);
    for (var i = 0; i < 60; i++) {
      await Future.delayed(Duration(seconds: 1));
      if (paymentResponse != null) {
        if (paymentResponse is PaymentSuccessResponse) {
          var successResponse = await verifyPayment(paymentResponse);
          return successResponse;
        }
        if (paymentResponse is PaymentFailureResponse) {
          debugPrint("Error ${paymentResponse.message}");
          return AppResponse.named(error: "${paymentResponse.message}");
        }
      }
    }
    return AppResponse.named(error: "Timeout Exception");
  }

  Future<AppResponse<String>> placeOrder(Amount amount) async {
    try {
      String url = paymentUrl + "/orders/";
      var json = {
        "amount": amount.amount.toInt() * 100,
        "currency": amount.currency
      };
      var response = await client.post(url, body: jsonEncode(json));
      var result = jsonDecode(response.body);
      var orderId = result["order_id"];
      return AppResponse.named(data: orderId);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<AppResponse<bool>> verifyPayment(
      PaymentSuccessResponse paymentResponse) async {
    try {
      String url = paymentUrl + "/orders/verify";
      var json = {
        "order_id": paymentResponse.orderId,
        "payment_id": paymentResponse.paymentId,
        "signature": paymentResponse.signature
      };
      var response = await client.post(url, body: jsonEncode(json));
      return AppResponse.named(data: response.statusCode == 200);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<String>> addMethod(PaymentMethod method) {
    // TODO: implement addMethod
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<PaymentBalance>> getBalance() async {
    try {
      String url = paymentUrl + "/balances/";
      var response = await client.get(
        url,
      );
      var json = jsonDecode(response.body);
      var balances = List<Map<String, dynamic>>.from(json["balances"] ?? []);
      debugPrint("Balances $balances");
      String currency = await getCurrency();
      if (balances.length > 0) {
        var amounts = balances
            .map((balance) => Amount.named(
                // TODO
                amount: (balance["balance"] ?? 0) / 100,
                currency: balance["currency"]))
            .toList();
        String currency = await getCurrency();
        int index =
            amounts.indexWhere((element) => element.currency == currency);
        if (index < 0) {
          return AppResponse.named(
              data: PaymentBalance.named(
                  amount: Amount.named(amount: 0, currency: currency)));
        }
        var amount = amounts[index];
        var paymentBalance = PaymentBalance.named(amount: amount);
        return AppResponse.named(data: paymentBalance);
      } else {
        return AppResponse.named(
            data: PaymentBalance.named(
                amount: Amount.named(amount: 0, currency: currency)));
      }
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Server error");
    }
  }

  @override
  Future<AppResponse<List<PaymentBalance>>> getMultipleBalances() async {
    try {
      String url = paymentUrl + "/balances/";
      var response = await client.get(
        url,
      );
      var json = jsonDecode(response.body);
      var balances = List<Map<String, dynamic>>.from(json["balances"] ?? []);
      debugPrint("Balances $balances");
      var amounts = balances
          .map((balance) => Amount.named(
              amount: (balance["balance"] ?? 0) / 100,
              currency: balance["currency"]))
          .toList();
      if (amounts.length == 0) {
        amounts.add(Amount.named(currency: await getCurrency(), amount: 0));
      }
      return AppResponse.named(
          data: amounts.map((e) => PaymentBalance.named(amount: e)).toList());
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Server error");
    }
  }

  @override
  Future<AppResponse<List<PaymentTransaction>>> getTransactions() async {
    try {
      String url = paymentUrl + "/history/";
      var response = await client.get(url);
      var json = jsonDecode(response.body);
      var transactions =
          List<Map<String, dynamic>>.from(json["payments"] ?? []);
      PaymentTransaction Function(Map<String, dynamic> json) fromJson =
          (Map<String, dynamic> json) {
        var newJson = json;
        debugPrint("Json $json");
        newJson["userId"] = json["user_id"];
        newJson["isCredited"] = json["is_credited"];
        newJson["recordedAt"] = json["recorded_at"];
        newJson["amount"] = json["amount"] ~/ 100;
        return PaymentTransaction.fromJson(newJson);
      };
      var result = List<PaymentTransaction>.from(
          transactions.map((e) => fromJson(e)).toList());
      return AppResponse.named(data: result);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  Future<String> getCurrency() async {
    try {
      debugPrint("Currency Get");
      String url = "/info/currency";
      var response = await client.get(url);
      var data = jsonDecode(response.body);
      debugPrint("Currency $data");
      String currency = data["currency"];
      return currency;
    } catch (e) {
      debugPrint("Error $e");
      return "USD";
    }
  }

  @override
  Future<AppResponse<List<PaymentMethod>>> getMethods() {
    // TODO: implement getMethods
    throw UnimplementedError();
  }
}
